<?php

/**
 * Copyright (c) 2016, BitKassa B.V.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * @category    BitKassa
 * @package     BitKassa_Bitcoin
 * @author      BitKassa (info@bitkassa.nl) / Mollie B.V. (info@mollie.nl)
 * @version     v1.0 (based on Mollie_Ideal v4.7)
 * @copyright   Copyright (c) 2012 Mollie B.V. (http://www.mollie.nl)
 * @license     http://www.opensource.org/licenses/bsd-license.php  Berkeley Software Distribution License (BSD-License 2)
 *
 **/

class ControllerPaymentBitKassaBitcoin extends Controller {
	// Initialize var(s)
	private $error = array();

	/**
	 * This method is executed by OpenCart when the Payment module is installed from the admin. It will create the
	 * required tables.
	 *
	 * @return void
	 */
	public function install () {
		$this->db->query(
			sprintf(
				"CREATE TABLE IF NOT EXISTS `%sbitkassa_payments` (
					`order_id` int(11) unsigned NOT NULL,
					`transaction_id` varchar(128) NOT NULL,
					`status` varchar(32) NOT NULL,
					PRIMARY KEY (`order_id`),
					UNIQUE KEY `transaction_id` (`transaction_id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8",
				DB_PREFIX
			));
	}

	/**
	 * The method is executed by OpenCart when the Payment module is uninstalled from the admin.
	 *  It will drop bitkassa payment table.
	 *
	 * @return void
	 */
	public function uninstall () {
		$this->db->query(sprintf(
			"DROP TABLE IF EXISTS `%sbitkassa_payments`",
			DB_PREFIX));
	}

	/**
	 * The backend for BitKassa
	 */
	public function index() {
		// Load essential models
		$this->load->language('payment/bitkassa_bitcoin');
		$this->load->model('setting/setting');
		$this->load->model('setting/store');
		$this->load->model('localisation/order_status');

		$this->document->setTitle($this->language->get('heading_title'));

		// Call validate method on POST
		if (($this->request->server['REQUEST_METHOD'] == 'POST') && ($this->validate())) {
			$this->load->model('setting/setting');
			$this->model_setting_setting->editSetting('bitcoin', $this->request->post);
			$this->session->data['success'] = $this->language->get('text_success');
			$this->redirect($this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL'));
		}

		// Set data for template
		$this->data['heading_title']          = $this->language->get('heading_title');

		$this->data['text_enabled']           = $this->language->get('text_enabled');
		$this->data['text_disabled']          = $this->language->get('text_disabled');
		$this->data['text_yes']               = $this->language->get('text_yes');
		$this->data['text_no']                = $this->language->get('text_no');
		$this->data['text_none']              = $this->language->get('text_none');

		$this->data['entry_status']           = $this->language->get('entry_status');
		$this->data['entry_merchantkey']      = $this->language->get('entry_merchantkey');
		$this->data['entry_description']      = $this->language->get('entry_description');
		$this->data['entry_total']            = $this->language->get('entry_total');

		$this->data['order_statuses']         = $this->model_localisation_order_status->getOrderStatuses();
		$this->data['entry_failed_status']    = $this->language->get('entry_failed_status');
		$this->data['entry_cancelled_status'] = $this->language->get('entry_cancelled_status');
		$this->data['entry_pending_status']   = $this->language->get('entry_pending_status');
		$this->data['entry_expired_status']   = $this->language->get('entry_expired_status');
		$this->data['entry_paid_status']      = $this->language->get('entry_paid_status');
		$this->data['entry_confirmed_status'] = $this->language->get('entry_confirmed_status');

		$this->data['entry_sort_order']       = $this->language->get('entry_sort_order');
		$this->data['entry_support']          = $this->language->get('entry_support');
		$this->data['entry_status']           = $this->language->get('entry_status');
		$this->data['entry_mstatus']          = $this->_checkModuleStatus();
		$this->data['entry_module']           = $this->language->get('entry_module');
		$this->data['entry_version']          = $this->language->get('entry_version');

		$this->data['button_save']            = $this->language->get('button_save');
		$this->data['button_cancel']          = $this->language->get('button_cancel');

		$this->data['tab_general']            = $this->language->get('tab_general');

		// If errors show the error
		if (isset($this->error['warning'])) {
			$this->data['error_warning'] = $this->error['warning'];
		} else {
			$this->data['error_warning'] = '';
		}
		if (isset($this->error['merchantkey'])) {
			$this->data['error_merchantkey'] = $this->error['merchantkey'];
		} else {
			$this->data['error_merchantkey'] = '';
		}
		if (isset($this->error['description'])) {
			$this->data['error_description'] = $this->error['description'];
		} else {
			$this->data['error_description'] = '';
		}
		if (isset($this->error['total'])) {
			$this->data['error_total'] = $this->error['total'];
		} else {
			$this->data['error_total'] = '';
		}

		// Breadcrumbs
		$this->data['breadcrumbs'] = array();

		$this->data['breadcrumbs'][] = array(
			'href' => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
			'text' => $this->language->get('text_home'),
			'separator' => FALSE
		);

		$this->data['breadcrumbs'][] = array(
			'href' => $this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL'),
			'text' => $this->language->get('text_payment'),
			'separator' => ' :: '
		);

		$this->data['breadcrumbs'][] = array(
			'href' => $this->url->link('payment/bitkassa_bitcoin', 'token=' . $this->session->data['token'], 'SSL'),
			'text' => $this->language->get('heading_title'),
			'separator' => ' :: '
		);

		// Form action url
		$this->data['action'] = $this->url->link('payment/bitkassa_bitcoin', 'token=' . $this->session->data['token'], 'SSL');
		$this->data['cancel'] = $this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL');

		// Post data
		if (isset($this->request->post['bitkassa_bitcoin_status'])) {
			$this->data['bitkassa_bitcoin_status'] = $this->request->post['bitkassa_bitcoin_status'];
		} else {
			$this->data['bitkassa_bitcoin_status'] = $this->config->get('bitkassa_bitcoin_status');
		}
		if (isset($this->request->post['bitkassa_bitcoin_merchantkey'])) {
			$this->data['bitkassa_bitcoin_merchantkey'] = $this->request->post['bitkassa_bitcoin_merchantkey'];
		} else {
			$this->data['bitkassa_bitcoin_merchantkey'] = $this->config->get('bitkassa_bitcoin_merchantkey');
		}
		if (isset($this->request->post['bitkassa_bitcoin_description'])) {
			$this->data['bitkassa_bitcoin_description'] = $this->request->post['bitkassa_bitcoin_description'];
		} else {
			$this->data['bitkassa_bitcoin_description'] = $this->config->get('bitkassa_bitcoin_description');
		}
		if (isset($this->request->post['bitkassa_bitcoin_failed_status_id'])) {
			$this->data['bitkassa_bitcoin_failed_status_id'] = $this->request->post['bitkassa_bitcoin_failed_status_id'];
		} else {
			$this->data['bitkassa_bitcoin_failed_status_id'] = $this->config->get('bitkassa_bitcoin_failed_status_id');
		}
		if (isset($this->request->post['bitkassa_bitcoin_cancelled_status_id'])) {
			$this->data['bitkassa_bitcoin_cancelled_status_id'] = $this->request->post['bitkassa_bitcoin_cancelled_status_id'];
		} else {
			$this->data['bitkassa_bitcoin_cancelled_status_id'] = $this->config->get('bitkassa_bitcoin_cancelled_status_id');
		}
		if (isset($this->request->post['bitkassa_bitcoin_expired_status_id'])) {
			$this->data['bitkassa_bitcoin_expired_status_id'] = $this->request->post['bitkassa_bitcoin_expired_status_id'];
		} else {
			$this->data['bitkassa_bitcoin_expired_status_id'] = $this->config->get('bitkassa_bitcoin_expired_status_id');
		}
		if (isset($this->request->post['bitkassa_bitcoin_pending_status_id'])) {
			$this->data['bitkassa_bitcoin_pending_status_id'] = $this->request->post['bitkassa_bitcoin_pending_status_id'];
		} else {
			$this->data['bitkassa_bitcoin_pending_status_id'] = $this->config->get('bitkassa_bitcoin_pending_status_id');
		}
		if (isset($this->request->post['bitkassa_bitcoin_paid_status_id'])) {
			$this->data['bitkassa_bitcoin_paid_status_id'] = $this->request->post['bitkassa_bitcoin_paid_status_id'];
		} else {
			$this->data['bitkassa_bitcoin_paid_status_id'] = $this->config->get('bitkassa_bitcoin_paid_status_id');
		}
		if (isset($this->request->post['bitkassa_bitcoin_confirmed_status_id'])) {
			$this->data['bitkassa_bitcoin_confirmed_status_id'] = $this->request->post['bitkassa_bitcoin_confirmed_status_id'];
		} else {
			$this->data['bitkassa_bitcoin_confirmed_status_id'] = $this->config->get('bitkassa_bitcoin_confirmed_status_id');
		}
		if (isset($this->request->post['bitkassa_bitcoin_sort_order'])) {
			$this->data['bitkassa_bitcoin_sort_order'] = $this->request->post['bitkassa_bitcoin_sort_order'];
		} else {
			$this->data['bitkassa_bitcoin_sort_order'] = $this->config->get('bitkassa_bitcoin_sort_order');
		}

		// Return and Report URL are static. They must stay here else OpenCart deletes them...
		$this->data['bitkassa_bitcoin_returnurl'] = $this->config->get('bitkassa_bitcoin_returnurl');
		$this->data['bitkassa_bitcoin_reporturl'] = $this->config->get('bitkassa_bitcoin_reporturl');

		// Set template
		$this->template = 'payment/bitkassa_bitcoin.tpl';
		$this->children = array(
			'common/header',
			'common/footer'
		);

		$this->response->setOutput($this->render(TRUE), $this->config->get('config_compression'));
	}

	/**
	 * Check the post and check if the user has permission to edit the module settings
	 *
	 * @return bool
	 */
	private function validate() {
		if (!$this->user->hasPermission('modify', 'payment/bitkassa_bitcoin')) {
			$this->error['warning'] = $this->language->get('error_permission');
		}
		if (!$this->request->post['bitkassa_bitcoin_merchantkey']) {
			$this->error['merchantkey'] = $this->language->get('error_merchantkey');
		}
		if (!$this->request->post['bitkassa_bitcoin_description']) {
			$this->error['description'] = $this->language->get('error_description');
		}

		if (!$this->error) {
			return TRUE;
		} else {
			return FALSE;
		}
	}

	protected function _checkModuleStatus() {
		$needFiles = array();
		$modFiles  = array(
			DIR_APPLICATION.'controller/payment/bitkassa_bitcoin.php',
			DIR_APPLICATION.'language/english/payment/bitkassa_bitcoin.php',
			DIR_TEMPLATE.'payment/bitkassa_bitcoin.tpl',
			DIR_CATALOG.'controller/payment/bitkassa_bitcoin.php',
			DIR_CATALOG.'language/english/payment/bitkassa_bitcoin.php',
			DIR_CATALOG.'model/payment/bitkassa_bitcoin.php',
			DIR_CATALOG.'view/theme/default/template/payment/bitkassa_bitcoin_return.tpl',
			DIR_CATALOG.'view/theme/default/template/payment/bitkassa_bitcoin_start.tpl'
		);

		foreach ($modFiles as $file) {
			if(!file_exists($file)) {
				$needFiles[] = '<span style="color:red">'.$file.'</span>';
			}
		}

		if (count($needFiles) > 0) {
			return $needFiles;
		} else {
			return '<span style="color:green">Ok!</span>';
		}

		return NULL;
	}

}
